﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Media;
using System.Windows;
using System.Windows.Controls;
using System.Diagnostics;

namespace WpfApplication1
{
    // WPF Grid 에 표처럼 Border 를 그려보자
    // ; https://blog.lunapiece.net/posts/WPF-Border-Grid/
    public class CustomGrid : Grid
    {
        public Thickness CellPadding
        {
            get
            {
                return (Thickness)this.GetValue(CellPaddingProperty);
            }
            set
            {
                this.SetValue(CellPaddingProperty, value);
            }
        }

        public static readonly DependencyProperty CellPaddingProperty =
            DependencyProperty.Register("CellPadding", typeof(Thickness), typeof(CustomGrid),
                new FrameworkPropertyMetadata(new Thickness(0.0, 0.0, 0.0, 0.0), FrameworkPropertyMetadataOptions.AffectsArrange,
                OnCellPaddingChanged)
            );

        static void OnCellPaddingChanged(DependencyObject dependencyObject, DependencyPropertyChangedEventArgs eventArgs)
        {
            CustomGrid grid = (dependencyObject as CustomGrid);
            foreach (UIElement uiElement in grid.Children)
            {
                ApplyMargin(grid, uiElement);
            }
        }

        protected override void OnVisualChildrenChanged(DependencyObject visualAdded, DependencyObject visualRemoved)
        {
            FrameworkElement childElement = visualAdded as FrameworkElement;
            ApplyMargin(this, childElement);

            base.OnVisualChildrenChanged(visualAdded, visualRemoved);
        }

        public static void ApplyMargin(CustomGrid PaddingGrid, UIElement element)
        {
            FrameworkElement childElement = element as FrameworkElement;
            Thickness cellPadding = PaddingGrid.CellPadding;

            CustomGrid childPaddingGrid = element as CustomGrid;
            if (childPaddingGrid != null)
            {
                childPaddingGrid.CellPadding = cellPadding;
            }
            else
            {
                if (childElement != null)
                {
                    childElement.Margin = cellPadding;
                }
            }
        }

        Pen line = new Pen(Brushes.Black, 2);
        protected override void OnRender(System.Windows.Media.DrawingContext dc)
        {
            base.OnRender(dc);

            CustomGrid customGrid = this.Parent as CustomGrid;
            if (customGrid == null)
            {
                dc.DrawRectangle(null, line, new Rect(0, 0, this.ActualWidth, this.ActualHeight));
            }

            double linePoint = 0;
            double posFrom = 0.0;
            double posTo = 0.0;

            int rowCount = Math.Max(this.RowDefinitions.Count, 1);
            int columnCount = Math.Max(this.ColumnDefinitions.Count, 1);

            bool[,] rowCellIgnoreStatus;
            bool[,] columnCellIgnoreStatus;

            GetRowLineCellStatus(rowCount, columnCount, out rowCellIgnoreStatus, out columnCellIgnoreStatus);

            if (this.ColumnDefinitions.Count != 0)
            {
                for (int row = 0; row < rowCount - 1; row++)
                {
                    var r = this.RowDefinitions[row];

                    linePoint += r.ActualHeight;
                    for (int column = 0; column < columnCount; column++)
                    {
                        bool ignoreLine = rowCellIgnoreStatus[row + 1, column];
                        posTo += this.ColumnDefinitions[column].ActualWidth;

                        if (ignoreLine == false)
                        {
                            dc.DrawLine(line, new Point(posFrom, linePoint), new Point(posTo, linePoint));
                        }

                        posFrom = posTo;
                    }

                    posFrom = 0.0;
                    posTo = 0.0;
                }
            }

            linePoint = 0;
            posFrom = 0.0;
            posTo = 0.0;

            if (this.RowDefinitions.Count != 0)
            {
                for (int column = 0; column < columnCount - 1; column++)
                {
                    var r = this.ColumnDefinitions[column];

                    linePoint += r.ActualWidth;
                    for (int row = 0; row < rowCount; row++)
                    {
                        bool ignoreLine = columnCellIgnoreStatus[row, column + 1];
                        posTo += this.RowDefinitions[row].ActualHeight;

                        if (ignoreLine == false)
                        {
                            dc.DrawLine(line, new Point(linePoint, posFrom), new Point(linePoint, posTo));
                        }

                        posFrom = posTo;
                    }

                    posTo = 0.0;
                    posFrom = 0.0;
                }
            }
        }

        private void GetRowLineCellStatus(int rowCount, int columnCount, out bool[,] rowCellIgnoreStatus, out bool[,] columnCellIgnoreStatus)
        {
            rowCellIgnoreStatus = new bool[rowCount, columnCount];
            columnCellIgnoreStatus = new bool[rowCount, columnCount];

            foreach (UIElement element in this.Children)
            {
                int row = Grid.GetRow(element);
                int column = Grid.GetColumn(element);

                int spanCount = Grid.GetColumnSpan(element);

                for (int span = 1; span < spanCount; span++)
                {
                    try
                    {
                        columnCellIgnoreStatus[row, column + span] = true;
                    }
                    catch (Exception)
                    {
                        //
                    }
                }

                spanCount = Grid.GetRowSpan(element);
                for (int span = 1; span < spanCount; span++)
                {
                    rowCellIgnoreStatus[row + span, column] = true;
                }
            }
        }
    }  
}
